@@ -1,7 +1,6 @@
 //! The typed heart of every Series column.
 use crate::prelude::*;
 use arrow::{array::*, bitmap::Bitmap};
-use itertools::Itertools;
 use polars_arrow::prelude::ValueSize;
 use std::marker::PhantomData;
 use std::sync::Arc;
@@ -49,7 +48,7 @@ use polars_arrow::prelude::*;
 
 #[cfg(feature = "dtype-categorical")]
 use crate::chunked_array::categorical::RevMapping;
-use crate::utils::{slice_offsets, CustomIterTools};
+use crate::utils::CustomIterTools;
 use std::mem;
 
 #[cfg(not(feature = "dtype-categorical"))]
@@ -155,6 +154,8 @@ pub struct ChunkedArray<T> {
     /// third bit dtype list: fast_explode
     ///     - unset: unknown or not all arrays have at least one value
     ///     - set: all list arrays are filled (this allows for cheap explode)
+    /// fourth bit: original local categorical
+    ///             meaning that n_unique is the same as the cat map length
     pub(crate) bit_settings: u8,
 }
 
@@ -210,7 +211,11 @@ impl<T> ChunkedArray<T> {
     /// Shrink the capacity of this array to fit it's length.
     pub fn shrink_to_fit(&mut self) {
         self.chunks = vec![arrow::compute::concatenate::concatenate(
-            self.chunks.iter().map(|a| &**a).collect_vec().as_slice(),
+            self.chunks
+                .iter()
+                .map(|a| &**a)
+                .collect::<Vec<_>>()
+                .as_slice(),
         )
         .unwrap()
         .into()];
@@ -232,7 +237,7 @@ impl<T> ChunkedArray<T> {
         } else {
             use DataType::*;
             match (self.dtype(), series.dtype()) {
-                (Int64, Datetime) | (Int32, Date) => {
+                (Int64, Datetime(_, _)) | (Int64, Duration(_)) | (Int32, Date) => {
                     &*(series_trait as *const dyn SeriesTrait as *const ChunkedArray<T>)
                 }
                 _ => panic!(
@@ -262,16 +267,6 @@ impl<T> ChunkedArray<T> {
         }
     }
 
-    /// Combined length of all the chunks.
-    pub fn len(&self) -> usize {
-        self.chunks.iter().fold(0, |acc, arr| acc + arr.len())
-    }
-
-    /// Check if ChunkedArray is empty.
-    pub fn is_empty(&self) -> bool {
-        self.len() == 0
-    }
-
     /// Unique id representing the number of chunks
     pub fn chunk_id(&self) -> ChunkIdIter {
         self.chunks.iter().map(|chunk| chunk.len())
@@ -293,17 +288,12 @@ impl<T> ChunkedArray<T> {
         self.chunks.iter().map(|arr| arr.null_count()).sum()
     }
 
-    /// Take a view of top n elements
-    pub fn limit(&self, num_elements: usize) -> Self {
-        self.slice(0, num_elements)
-    }
-
     /// Append arrow array in place.
     ///
     /// ```rust
     /// # use polars_core::prelude::*;
-    /// let mut array = Int32Chunked::new_from_slice("array", &[1, 2]);
-    /// let array_2 = Int32Chunked::new_from_slice("2nd", &[3]);
+    /// let mut array = Int32Chunked::new("array", &[1, 2]);
+    /// let array_2 = Int32Chunked::new("2nd", &[3]);
     ///
     /// array.append(&array_2);
     /// assert_eq!(Vec::from(&array), [Some(1), Some(2), Some(3)])
@@ -340,41 +330,6 @@ impl<T> ChunkedArray<T> {
         }
     }
 
-    /// Slice the array. The chunks are reallocated the underlying data slices are zero copy.
-    ///
-    /// When offset is negative it will be counted from the end of the array.
-    /// This method will never error,
-    /// and will slice the best match when offset, or length is out of bounds
-    pub fn slice(&self, offset: i64, length: usize) -> Self {
-        let (raw_offset, slice_len) = slice_offsets(offset, length, self.len());
-
-        let mut remaining_length = slice_len;
-        let mut remaining_offset = raw_offset;
-        let mut new_chunks = vec![];
-
-        for chunk in &self.chunks {
-            let chunk_len = chunk.len();
-            if remaining_offset > 0 && remaining_offset >= chunk_len {
-                remaining_offset -= chunk_len;
-                continue;
-            }
-            let take_len;
-            if remaining_length + remaining_offset > chunk_len {
-                take_len = chunk_len - remaining_offset;
-            } else {
-                take_len = remaining_length;
-            }
-
-            new_chunks.push(chunk.slice(remaining_offset, take_len).into());
-            remaining_length -= take_len;
-            remaining_offset = 0;
-            if remaining_length == 0 {
-                break;
-            }
-        }
-        self.copy_with_chunks(new_chunks)
-    }
-
     /// Get a mask of the null values.
     pub fn is_null(&self) -> BooleanChunked {
         if !self.has_validity() {
@@ -390,7 +345,7 @@ impl<T> ChunkedArray<T> {
                     .unwrap_or_else(|| Bitmap::new_zeroed(arr.len()));
                 Arc::new(BooleanArray::from_data_default(bitmap, None)) as ArrayRef
             })
-            .collect_vec();
+            .collect::<Vec<_>>();
         BooleanChunked::new_from_chunks(self.name(), chunks)
     }
 
@@ -409,7 +364,7 @@ impl<T> ChunkedArray<T> {
                     .unwrap_or_else(|| !(&Bitmap::new_zeroed(arr.len())));
                 Arc::new(BooleanArray::from_data_default(bitmap, None)) as ArrayRef
             })
-            .collect_vec();
+            .collect::<Vec<_>>();
         BooleanChunked::new_from_chunks(self.name(), chunks)
     }
 
@@ -418,23 +373,6 @@ impl<T> ChunkedArray<T> {
         self.field.data_type()
     }
 
-    /// Get the head of the ChunkedArray
-    pub fn head(&self, length: Option<usize>) -> Self {
-        match length {
-            Some(len) => self.slice(0, std::cmp::min(len, self.len())),
-            None => self.slice(0, std::cmp::min(10, self.len())),
-        }
-    }
-
-    /// Get the tail of the ChunkedArray
-    pub fn tail(&self, length: Option<usize>) -> Self {
-        let len = match length {
-            Some(len) => std::cmp::min(len, self.len()),
-            None => std::cmp::min(10, self.len()),
-        };
-        self.slice(-(len as i64), len)
-    }
-
     /// Name of the ChunkedArray.
     pub fn name(&self) -> &str {
         self.field.name()
@@ -520,8 +458,8 @@ impl<T> ChunkedArray<T>
 where
     T: PolarsNumericType,
 {
-    /// Create a new ChunkedArray by taking ownership of the AlignedVec. This operation is zero copy.
-    pub fn new_from_aligned_vec(name: &str, v: AlignedVec<T::Native>) -> Self {
+    /// Create a new ChunkedArray by taking ownership of the Vec. This operation is zero copy.
+    pub fn new_from_aligned_vec(name: &str, v: Vec<T::Native>) -> Self {
         let arr = to_array::<T>(v, None);
         Self::new_from_chunks(name, vec![arr])
     }
@@ -529,7 +467,7 @@ where
     /// Nullify values in slice with an existing null bitmap
     pub fn new_from_owned_with_null_bitmap(
         name: &str,
-        values: AlignedVec<T::Native>,
+        values: Vec<T::Native>,
         buffer: Option<Bitmap>,
     ) -> Self {
         let arr = to_array::<T>(values, buffer);
@@ -604,10 +542,13 @@ where
            + TrustedLen {
         // .copied was significantly slower in benchmark, next call did not inline?
         #[allow(clippy::map_clone)]
-        self.data_views()
-            .flatten()
-            .map(|v| *v)
-            .trust_my_length(self.len())
+        // we know the iterators len
+        unsafe {
+            self.data_views()
+                .flatten()
+                .map(|v| *v)
+                .trust_my_length(self.len())
+        }
     }
 }
 
@@ -656,14 +597,14 @@ impl ListChunked {
 }
 
 pub(crate) fn to_primitive<T: PolarsNumericType>(
-    values: AlignedVec<T::Native>,
+    values: Vec<T::Native>,
     validity: Option<Bitmap>,
 ) -> PrimitiveArray<T::Native> {
     PrimitiveArray::from_data(T::get_dtype().to_arrow(), values.into(), validity)
 }
 
 pub(crate) fn to_array<T: PolarsNumericType>(
-    values: AlignedVec<T::Native>,
+    values: Vec<T::Native>,
     validity: Option<Bitmap>,
 ) -> ArrayRef {
     Arc::new(to_primitive::<T>(values, validity))
@@ -680,19 +621,19 @@ pub(crate) mod test {
     use crate::prelude::*;
 
     pub(crate) fn get_chunked_array() -> Int32Chunked {
-        ChunkedArray::new_from_slice("a", &[1, 2, 3])
+        ChunkedArray::new("a", &[1, 2, 3])
     }
 
     #[test]
     fn test_sort() {
-        let a = Int32Chunked::new_from_slice("a", &[1, 9, 3, 2]);
+        let a = Int32Chunked::new("a", &[1, 9, 3, 2]);
         let b = a
             .sort(false)
             .into_iter()
             .map(|opt| opt.unwrap())
             .collect::<Vec<_>>();
         assert_eq!(b, [1, 2, 3, 9]);
-        let a = Utf8Chunked::new_from_slice("a", &["b", "a", "c"]);
+        let a = Utf8Chunked::new("a", &["b", "a", "c"]);
         let a = a.sort(false);
         let b = a.into_iter().collect::<Vec<_>>();
         assert_eq!(b, [Some("a"), Some("b"), Some("c")]);
@@ -728,10 +669,7 @@ pub(crate) mod test {
     fn filter() {
         let a = get_chunked_array();
         let b = a
-            .filter(&BooleanChunked::new_from_slice(
-                "filter",
-                &[true, false, false],
-            ))
+            .filter(&BooleanChunked::new("filter", &[true, false, false]))
             .unwrap();
         assert_eq!(b.len(), 1);
         assert_eq!(b.into_iter().next(), Some(Some(1)));
@@ -781,8 +719,8 @@ pub(crate) mod test {
 
     #[test]
     fn slice() {
-        let mut first = UInt32Chunked::new_from_slice("first", &[0, 1, 2]);
-        let second = UInt32Chunked::new_from_slice("second", &[3, 4, 5]);
+        let mut first = UInt32Chunked::new("first", &[0, 1, 2]);
+        let second = UInt32Chunked::new("second", &[3, 4, 5]);
         first.append(&second);
         assert_slice_equal(&first.slice(0, 3), &[0, 1, 2]);
         assert_slice_equal(&first.slice(0, 4), &[0, 1, 2, 3]);
@@ -800,7 +738,7 @@ pub(crate) mod test {
 
     #[test]
     fn sorting() {
-        let s = UInt32Chunked::new_from_slice("", &[9, 2, 4]);
+        let s = UInt32Chunked::new("", &[9, 2, 4]);
         let sorted = s.sort(false);
         assert_slice_equal(&sorted, &[2, 4, 9]);
         let sorted = s.sort(true);
@@ -827,29 +765,29 @@ pub(crate) mod test {
 
     #[test]
     fn reverse() {
-        let s = UInt32Chunked::new_from_slice("", &[1, 2, 3]);
+        let s = UInt32Chunked::new("", &[1, 2, 3]);
         // path with continuous slice
         assert_slice_equal(&s.reverse(), &[3, 2, 1]);
         // path with options
-        let s = UInt32Chunked::new_from_opt_slice("", &[Some(1), None, Some(3)]);
+        let s = UInt32Chunked::new("", &[Some(1), None, Some(3)]);
         assert_eq!(Vec::from(&s.reverse()), &[Some(3), None, Some(1)]);
-        let s = BooleanChunked::new_from_slice("", &[true, false]);
+        let s = BooleanChunked::new("", &[true, false]);
         assert_eq!(Vec::from(&s.reverse()), &[Some(false), Some(true)]);
 
-        let s = Utf8Chunked::new_from_slice("", &["a", "b", "c"]);
+        let s = Utf8Chunked::new("", &["a", "b", "c"]);
         assert_eq!(Vec::from(&s.reverse()), &[Some("c"), Some("b"), Some("a")]);
 
-        let s = Utf8Chunked::new_from_opt_slice("", &[Some("a"), None, Some("c")]);
+        let s = Utf8Chunked::new("", &[Some("a"), None, Some("c")]);
         assert_eq!(Vec::from(&s.reverse()), &[Some("c"), None, Some("a")]);
     }
 
     #[test]
     fn test_null_sized_chunks() {
-        let mut s = Float64Chunked::new_from_slice("s", &Vec::<f64>::new());
-        s.append(&Float64Chunked::new_from_slice("s2", &[1., 2., 3.]));
+        let mut s = Float64Chunked::new("s", &Vec::<f64>::new());
+        s.append(&Float64Chunked::new("s2", &[1., 2., 3.]));
         dbg!(&s);
 
-        let s = Float64Chunked::new_from_slice("s", &Vec::<f64>::new());
+        let s = Float64Chunked::new("s", &Vec::<f64>::new());
         dbg!(&s.into_iter().next());
     }
 
@@ -860,8 +798,7 @@ pub(crate) mod test {
         use crate::SINGLE_LOCK;
         let _lock = SINGLE_LOCK.lock();
         reset_string_cache();
-        let ca =
-            Utf8Chunked::new_from_opt_slice("", &[Some("foo"), None, Some("bar"), Some("ham")]);
+        let ca = Utf8Chunked::new("", &[Some("foo"), None, Some("bar"), Some("ham")]);
         let ca = ca.cast(&DataType::Categorical).unwrap();
         let ca = ca.categorical().unwrap();
         let v: Vec<_> = ca.into_iter().collect();
