@@ -2,6 +2,8 @@
 pub use crate::prelude::ChunkCompare;
 use crate::prelude::*;
 use arrow::array::ArrayRef;
+use polars_arrow::prelude::QuantileInterpolOptions;
+
 pub(crate) mod arithmetic;
 mod comparison;
 mod from;
@@ -15,7 +17,7 @@ pub mod unstable;
 
 use crate::chunked_array::ops::rolling_window::RollingOptions;
 #[cfg(feature = "rank")]
-use crate::prelude::unique::rank::{rank, RankMethod};
+use crate::prelude::unique::rank::rank;
 #[cfg(feature = "groupby_list")]
 use crate::utils::Wrap;
 use crate::utils::{split_ca, split_series};
@@ -80,7 +82,6 @@ use std::sync::Arc;
 ///
 /// ```
 /// # use polars_core::prelude::*;
-/// use itertools::Itertools;
 /// let s = Series::new("dollars", &[1, 2, 3]);
 /// let mask = s.equal(1);
 /// let valid = [true, false, false].iter();
@@ -130,6 +131,7 @@ use std::sync::Arc;
 ///
 /// ```
 #[derive(Clone)]
+#[must_use]
 pub struct Series(pub Arc<dyn SeriesTrait>);
 
 #[cfg(feature = "groupby_list")]
@@ -182,13 +184,13 @@ impl Series {
         Ok(self)
     }
 
-    /// Sort in place.
-    pub fn sort_in_place(&mut self, reverse: bool) -> &mut Self {
-        self.get_inner_mut().sort_in_place(reverse);
-        self
+    pub fn sort(&self, reverse: bool) -> Self {
+        self.sort_with(SortOptions {
+            descending: reverse,
+            ..Default::default()
+        })
     }
 
-    /// Rechunk and return a pointer to the start of the Series.
     /// Only implemented for numeric types
     pub fn as_single_ptr(&mut self) -> Result<usize> {
         self.get_inner_mut().as_single_ptr()
@@ -198,10 +200,16 @@ impl Series {
     pub fn cast(&self, dtype: &DataType) -> Result<Self> {
         self.0.cast(dtype)
     }
+
+    /// Compute the sum of all values in this Series.
     /// Returns `None` if the array is empty or only contains null values.
+    ///
+    /// If the [`DataType`] is one of `{Int8, UInt8, Int16, UInt16}` the `Series` is
+    /// first cast to `Int64` to prevent overflow issues.
+    ///
     /// ```
     /// # use polars_core::prelude::*;
-    /// let s = Series::new("days", [1, 2, 3].as_ref());
+    /// let s = Series::new("days", &[1, 2, 3]);
     /// assert_eq!(s.sum(), Some(6));
     /// ```
     pub fn sum<T>(&self) -> Option<T>
@@ -326,12 +334,14 @@ impl Series {
     /// * Date -> Int32
     /// * Datetime-> Int64
     /// * Time -> Int64
+    /// * Categorical -> UInt32
     ///
     pub fn to_physical_repr(&self) -> Cow<Series> {
         use DataType::*;
         match self.dtype() {
             Date => Cow::Owned(self.cast(&DataType::Int32).unwrap()),
-            Datetime | Time => Cow::Owned(self.cast(&DataType::Int64).unwrap()),
+            Datetime(_, _) | Duration(_) | Time => Cow::Owned(self.cast(&DataType::Int64).unwrap()),
+            Categorical => Cow::Owned(self.cast(&DataType::UInt32).unwrap()),
             _ => Cow::Borrowed(self),
         }
     }
@@ -428,47 +438,6 @@ impl Series {
         }
     }
 
-    /// Round underlying floating point array to given decimal.
-    #[cfg(feature = "round_series")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "round_series")))]
-    pub fn round(&self, decimals: u32) -> Result<Self> {
-        use num::traits::Pow;
-        if let Ok(ca) = self.f32() {
-            let multiplier = 10.0.pow(decimals as f32) as f32;
-            let s = ca
-                .apply(|val| (val * multiplier).round() / multiplier)
-                .into_series();
-            return Ok(s);
-        }
-        if let Ok(ca) = self.f64() {
-            let multiplier = 10.0.pow(decimals as f32) as f64;
-            let s = ca
-                .apply(|val| (val * multiplier).round() / multiplier)
-                .into_series();
-            return Ok(s);
-        }
-        Err(PolarsError::SchemaMisMatch(
-            format!("{:?} is not a floating point datatype", self.dtype()).into(),
-        ))
-    }
-
-    #[cfg(feature = "round_series")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "round_series")))]
-    /// Floor underlying floating point array to the lowest integers smaller or equal to the float value.
-    pub fn floor(&self) -> Result<Self> {
-        if let Ok(ca) = self.f32() {
-            let s = ca.apply(|val| val.floor()).into_series();
-            return Ok(s);
-        }
-        if let Ok(ca) = self.f64() {
-            let s = ca.apply(|val| val.floor()).into_series();
-            return Ok(s);
-        }
-        Err(PolarsError::SchemaMisMatch(
-            format!("{:?} is not a floating point datatype", self.dtype()).into(),
-        ))
-    }
-
     #[cfg(feature = "dot_product")]
     #[cfg_attr(docsrs, doc(cfg(feature = "dot_product")))]
     pub fn dot(&self, other: &Series) -> Option<f64> {
@@ -482,6 +451,18 @@ impl Series {
         UInt64Chunked::new_from_aligned_vec(self.name(), self.0.vec_hash(build_hasher))
     }
 
+    /// Get the sum of the Series as a new Series of length 1.
+    ///
+    /// If the [`DataType`] is one of `{Int8, UInt8, Int16, UInt16}` the `Series` is
+    /// first cast to `Int64` to prevent overflow issues.
+    pub fn sum_as_series(&self) -> Series {
+        use DataType::*;
+        match self.dtype() {
+            Int8 | UInt8 | Int16 | UInt16 => self.cast(&Int64).unwrap().sum_as_series(),
+            _ => self._sum_as_series(),
+        }
+    }
+
     /// Get an array with the cumulative max computed at every element
     #[cfg_attr(docsrs, doc(cfg(feature = "cum_agg")))]
     pub fn cummax(&self, _reverse: bool) -> Series {
@@ -509,13 +490,46 @@ impl Series {
     }
 
     /// Get an array with the cumulative sum computed at every element
+    ///
+    /// If the [`DataType`] is one of `{Int8, UInt8, Int16, UInt16}` the `Series` is
+    /// first cast to `Int64` to prevent overflow issues.
     #[cfg_attr(docsrs, doc(cfg(feature = "cum_agg")))]
-    pub fn cumsum(&self, _reverse: bool) -> Series {
+    #[allow(unused_variables)]
+    pub fn cumsum(&self, reverse: bool) -> Series {
         #[cfg(feature = "cum_agg")]
         {
+            use DataType::*;
             match self.dtype() {
-                DataType::Boolean => self.cast(&DataType::UInt32).unwrap()._cumsum(_reverse),
-                _ => self._cumsum(_reverse),
+                Boolean => self.cast(&DataType::UInt32).unwrap().cumsum(reverse),
+                Int8 | UInt8 | Int16 | UInt16 => {
+                    let s = self.cast(&Int64).unwrap();
+                    s.cumsum(reverse)
+                }
+                Int32 => {
+                    let ca = self.i32().unwrap();
+                    ca.cumsum(reverse).into_series()
+                }
+                UInt32 => {
+                    let ca = self.u32().unwrap();
+                    ca.cumsum(reverse).into_series()
+                }
+                UInt64 => {
+                    let ca = self.u64().unwrap();
+                    ca.cumsum(reverse).into_series()
+                }
+                Int64 => {
+                    let ca = self.i64().unwrap();
+                    ca.cumsum(reverse).into_series()
+                }
+                Float32 => {
+                    let ca = self.f32().unwrap();
+                    ca.cumsum(reverse).into_series()
+                }
+                Float64 => {
+                    let ca = self.f64().unwrap();
+                    ca.cumsum(reverse).into_series()
+                }
+                dt => panic!("cumsum not supported for dtype: {:?}", dt),
             }
         }
         #[cfg(not(feature = "cum_agg"))]
@@ -525,13 +539,38 @@ impl Series {
     }
 
     /// Get an array with the cumulative product computed at every element
+    ///
+    /// If the [`DataType`] is one of `{Int8, UInt8, Int16, UInt16, Int32, UInt32}` the `Series` is
+    /// first cast to `Int64` to prevent overflow issues.
     #[cfg_attr(docsrs, doc(cfg(feature = "cum_agg")))]
-    pub fn cumprod(&self, _reverse: bool) -> Series {
+    #[allow(unused_variables)]
+    pub fn cumprod(&self, reverse: bool) -> Series {
         #[cfg(feature = "cum_agg")]
         {
+            use DataType::*;
             match self.dtype() {
-                DataType::Boolean => self.cast(&DataType::UInt32).unwrap()._cumprod(_reverse),
-                _ => self._cumprod(_reverse),
+                Boolean => self.cast(&DataType::Int64).unwrap().cumprod(reverse),
+                Int8 | UInt8 | Int16 | UInt16 | Int32 | UInt32 => {
+                    let s = self.cast(&Int64).unwrap();
+                    s.cumprod(reverse)
+                }
+                Int64 => {
+                    let ca = self.i64().unwrap();
+                    ca.cumprod(reverse).into_series()
+                }
+                UInt64 => {
+                    let ca = self.u64().unwrap();
+                    ca.cumprod(reverse).into_series()
+                }
+                Float32 => {
+                    let ca = self.f32().unwrap();
+                    ca.cumprod(reverse).into_series()
+                }
+                Float64 => {
+                    let ca = self.f64().unwrap();
+                    ca.cumprod(reverse).into_series()
+                }
+                dt => panic!("cumprod not supported for dtype: {:?}", dt),
             }
         }
         #[cfg(not(feature = "cum_agg"))]
@@ -540,6 +579,42 @@ impl Series {
         }
     }
 
+    /// Get the product of an array.
+    ///
+    /// If the [`DataType`] is one of `{Int8, UInt8, Int16, UInt16}` the `Series` is
+    /// first cast to `Int64` to prevent overflow issues.
+    #[cfg_attr(docsrs, doc(cfg(feature = "product")))]
+    pub fn product(&self) -> Series {
+        #[cfg(feature = "product")]
+        {
+            use DataType::*;
+            match self.dtype() {
+                Boolean => self.cast(&DataType::Int64).unwrap().product(),
+                Int8 | UInt8 | Int16 | UInt16 => {
+                    let s = self.cast(&Int64).unwrap();
+                    s.product()
+                }
+                Int64 => {
+                    let ca = self.i64().unwrap();
+                    ca.prod_as_series()
+                }
+                Float32 => {
+                    let ca = self.f32().unwrap();
+                    ca.prod_as_series()
+                }
+                Float64 => {
+                    let ca = self.f64().unwrap();
+                    ca.prod_as_series()
+                }
+                dt => panic!("cumprod not supported for dtype: {:?}", dt),
+            }
+        }
+        #[cfg(not(feature = "product"))]
+        {
+            panic!("activate 'product' feature")
+        }
+    }
+
     /// Apply a rolling variance to a Series. See:
     #[cfg_attr(docsrs, doc(cfg(feature = "rolling_window")))]
     pub fn rolling_var(&self, _options: RollingOptions) -> Result<Series> {
@@ -567,7 +642,7 @@ impl Series {
     }
 
     /// Apply a rolling mean to a Series. See:
-    /// [ChunkedArray::rolling_mean](crate::prelude::ChunkWindow::rolling_mean).
+    /// [ChunkedArray::rolling_mean]
     #[cfg_attr(docsrs, doc(cfg(feature = "rolling_window")))]
     pub fn rolling_mean(&self, _options: RollingOptions) -> Result<Series> {
         #[cfg(feature = "rolling_window")]
@@ -580,7 +655,7 @@ impl Series {
         }
     }
     /// Apply a rolling sum to a Series. See:
-    /// [ChunkedArray::rolling_sum](crate::prelude::ChunkWindow::rolling_sum).
+    /// [ChunkedArray::rolling_sum]
     #[cfg_attr(docsrs, doc(cfg(feature = "rolling_window")))]
     pub fn rolling_sum(&self, _options: RollingOptions) -> Result<Series> {
         #[cfg(feature = "rolling_window")]
@@ -592,8 +667,39 @@ impl Series {
             panic!("activate 'rolling_window' feature")
         }
     }
+    /// Apply a rolling median to a Series. See:
+    /// [ChunkedArray::rolling_median](crate::prelude::ChunkWindow::rolling_median).
+    #[cfg_attr(docsrs, doc(cfg(feature = "rolling_window")))]
+    pub fn rolling_median(&self, _options: RollingOptions) -> Result<Series> {
+        #[cfg(feature = "rolling_window")]
+        {
+            self._rolling_median(_options)
+        }
+        #[cfg(not(feature = "rolling_window"))]
+        {
+            panic!("activate 'rolling_window' feature")
+        }
+    }
+    /// Apply a rolling quantile to a Series. See:
+    /// [ChunkedArray::rolling_quantile](crate::prelude::ChunkWindow::rolling_quantile).
+    #[cfg_attr(docsrs, doc(cfg(feature = "rolling_window")))]
+    pub fn rolling_quantile(
+        &self,
+        _quantile: f64,
+        _interpolation: QuantileInterpolOptions,
+        _options: RollingOptions,
+    ) -> Result<Series> {
+        #[cfg(feature = "rolling_window")]
+        {
+            self._rolling_quantile(_quantile, _interpolation, _options)
+        }
+        #[cfg(not(feature = "rolling_window"))]
+        {
+            panic!("activate 'rolling_window' feature")
+        }
+    }
     /// Apply a rolling min to a Series. See:
-    /// [ChunkedArray::rolling_min](crate::prelude::ChunkWindow::rolling_min).
+    /// [ChunkedArray::rolling_min]
     #[cfg_attr(docsrs, doc(cfg(feature = "rolling_window")))]
     pub fn rolling_min(&self, _options: RollingOptions) -> Result<Series> {
         #[cfg(feature = "rolling_window")]
@@ -606,7 +712,7 @@ impl Series {
         }
     }
     /// Apply a rolling max to a Series. See:
-    /// [ChunkedArray::rolling_max](crate::prelude::ChunkWindow::rolling_max).
+    /// [ChunkedArray::rolling_max]
     #[cfg_attr(docsrs, doc(cfg(feature = "rolling_window")))]
     pub fn rolling_max(&self, _options: RollingOptions) -> Result<Series> {
         #[cfg(feature = "rolling_window")]
@@ -621,8 +727,8 @@ impl Series {
 
     #[cfg(feature = "rank")]
     #[cfg_attr(docsrs, doc(cfg(feature = "rank")))]
-    pub fn rank(&self, method: RankMethod) -> Series {
-        rank(self, method)
+    pub fn rank(&self, options: RankOptions) -> Series {
+        rank(self, options.method, options.descending)
     }
 
     /// Cast throws an error if conversion had overflows
@@ -636,28 +742,124 @@ impl Series {
                     self.dtype(),
                     data_type
                 )
-                .into(),
+                    .into(),
             ))
         } else {
             Ok(s)
         }
     }
+
     #[cfg(feature = "dtype-time")]
     pub(crate) fn into_time(self) -> Series {
-        self.i64()
-            .expect("impl error")
-            .clone()
-            .into_time()
-            .into_series()
+        match self.dtype() {
+            DataType::Int64 => self.i64().unwrap().clone().into_time().into_series(),
+            DataType::Time => self
+                .time()
+                .unwrap()
+                .as_ref()
+                .clone()
+                .into_time()
+                .into_series(),
+            dt => panic!("date not implemented for {:?}", dt),
+        }
     }
 
     pub(crate) fn into_date(self) -> Series {
         match self.dtype() {
             #[cfg(feature = "dtype-date")]
             DataType::Int32 => self.i32().unwrap().clone().into_date().into_series(),
+            #[cfg(feature = "dtype-date")]
+            DataType::Date => self
+                .date()
+                .unwrap()
+                .as_ref()
+                .clone()
+                .into_date()
+                .into_series(),
+            dt => panic!("date not implemented for {:?}", dt),
+        }
+    }
+    pub(crate) fn into_datetime(self, timeunit: TimeUnit, tz: Option<TimeZone>) -> Series {
+        match self.dtype() {
             #[cfg(feature = "dtype-datetime")]
-            DataType::Int64 => self.i64().unwrap().clone().into_date().into_series(),
-            _ => unreachable!(),
+            DataType::Int64 => self
+                .i64()
+                .unwrap()
+                .clone()
+                .into_datetime(timeunit, tz)
+                .into_series(),
+            #[cfg(feature = "dtype-datetime")]
+            DataType::Datetime(_, _) => self
+                .datetime()
+                .unwrap()
+                .as_ref()
+                .clone()
+                .into_datetime(timeunit, tz)
+                .into_series(),
+            dt => panic!("into_datetime not implemented for {:?}", dt),
+        }
+    }
+
+    pub(crate) fn into_duration(self, timeunit: TimeUnit) -> Series {
+        match self.dtype() {
+            #[cfg(feature = "dtype-duration")]
+            DataType::Int64 => self
+                .i64()
+                .unwrap()
+                .clone()
+                .into_duration(timeunit)
+                .into_series(),
+            #[cfg(feature = "dtype-duration")]
+            DataType::Duration(_) => self
+                .duration()
+                .unwrap()
+                .as_ref()
+                .clone()
+                .into_duration(timeunit)
+                .into_series(),
+            dt => panic!("into_duration not implemented for {:?}", dt),
+        }
+    }
+
+    /// Check if the underlying data is a logical type.
+    pub fn is_logical(&self) -> bool {
+        use DataType::*;
+        matches!(
+            self.dtype(),
+            Date | Duration(_) | Datetime(_, _) | Time | Categorical
+        )
+    }
+
+    /// Check if underlying physical data is numeric.
+    ///
+    /// Date types and Categoricals are also considered numeric.
+    pub fn is_numeric_physical(&self) -> bool {
+        // allow because it cannot be replaced when object feature is activated
+        #[allow(clippy::match_like_matches_macro)]
+        match self.dtype() {
+            DataType::Utf8 | DataType::List(_) | DataType::Boolean | DataType::Null => false,
+            #[cfg(feature = "object")]
+            DataType::Object(_) => false,
+            _ => true,
+        }
+    }
+
+    /// Check if underlying data is numeric
+    pub fn is_numeric(&self) -> bool {
+        // allow because it cannot be replaced when object feature is activated
+        #[allow(clippy::match_like_matches_macro)]
+        match self.dtype() {
+            DataType::Utf8
+            | DataType::List(_)
+            | DataType::Categorical
+            | DataType::Date
+            | DataType::Datetime(_, _)
+            | DataType::Duration(_)
+            | DataType::Boolean
+            | DataType::Null => false,
+            #[cfg(feature = "object")]
+            DataType::Object(_) => false,
+            _ => true,
         }
     }
 
@@ -680,11 +882,39 @@ impl Series {
             dt => {
                 return Err(PolarsError::InvalidOperation(
                     format!("abs not supportedd for series of type {:?}", dt).into(),
-                ))
+                ));
             }
         };
         Ok(out)
     }
+
+    #[cfg(feature = "private")]
+    // used for formatting
+    pub fn str_value(&self, index: usize) -> Cow<str> {
+        match self.0.get(index) {
+            AnyValue::Utf8(s) => Cow::Borrowed(s),
+            AnyValue::Null => Cow::Borrowed("null"),
+            #[cfg(feature = "dtype-categorical")]
+            AnyValue::Categorical(idx, rev) => Cow::Borrowed(rev.get(idx)),
+            av => Cow::Owned(format!("{}", av)),
+        }
+    }
+    /// Get the head of the Series.
+    pub fn head(&self, length: Option<usize>) -> Series {
+        match length {
+            Some(len) => self.slice(0, std::cmp::min(len, self.len())),
+            None => self.slice(0, std::cmp::min(10, self.len())),
+        }
+    }
+
+    /// Get the tail of the Series.
+    pub fn tail(&self, length: Option<usize>) -> Series {
+        let len = match length {
+            Some(len) => std::cmp::min(len, self.len()),
+            None => std::cmp::min(10, self.len()),
+        };
+        self.slice(-(len as i64), len)
+    }
 }
 
 impl Deref for Series {
@@ -714,7 +944,7 @@ where
     fn as_ref(&self) -> &ChunkedArray<T> {
         if &T::get_dtype() == self.dtype() ||
             // needed because we want to get ref of List no matter what the inner type is.
-            (matches!(T::get_dtype(), DataType::List(_)) && matches!(self.dtype(), DataType::List(_)) )
+            (matches!(T::get_dtype(), DataType::List(_)) && matches!(self.dtype(), DataType::List(_)))
         {
             unsafe { &*(self as *const dyn SeriesTrait as *const ChunkedArray<T>) }
         } else {
@@ -734,7 +964,7 @@ where
     fn as_mut(&mut self) -> &mut ChunkedArray<T> {
         if &T::get_dtype() == self.dtype() ||
             // needed because we want to get ref of List no matter what the inner type is.
-            (matches!(T::get_dtype(), DataType::List(_)) && matches!(self.dtype(), DataType::List(_)) )
+            (matches!(T::get_dtype(), DataType::List(_)) && matches!(self.dtype(), DataType::List(_)))
         {
             unsafe { &mut *(self as *mut dyn SeriesTrait as *mut ChunkedArray<T>) }
         } else {
@@ -755,7 +985,7 @@ mod test {
 
     #[test]
     fn cast() {
-        let ar = UInt32Chunked::new_from_slice("a", &[1, 2]);
+        let ar = UInt32Chunked::new("a", &[1, 2]);
         let s = ar.into_series();
         let s2 = s.cast(&DataType::Int64).unwrap();
 
@@ -766,10 +996,10 @@ mod test {
 
     #[test]
     fn new_series() {
-        Series::new("boolean series", &vec![true, false, true]);
-        Series::new("int series", &[1, 2, 3]);
-        let ca = Int32Chunked::new_from_slice("a", &[1, 2, 3]);
-        ca.into_series();
+        let _ = Series::new("boolean series", &vec![true, false, true]);
+        let _ = Series::new("int series", &[1, 2, 3]);
+        let ca = Int32Chunked::new("a", &[1, 2, 3]);
+        let _ = ca.into_series();
     }
 
     #[test]
@@ -777,7 +1007,7 @@ mod test {
         let array = UInt32Array::from_slice(&[1, 2, 3, 4, 5]);
         let array_ref: ArrayRef = Arc::new(array);
 
-        Series::try_from(("foo", array_ref)).unwrap();
+        let _ = Series::try_from(("foo", array_ref)).unwrap();
     }
 
     #[test]
@@ -809,9 +1039,9 @@ mod test {
     fn out_of_range_slice_does_not_panic() {
         let series = Series::new("a", &[1i64, 2, 3, 4, 5]);
 
-        series.slice(-3, 4);
-        series.slice(-6, 2);
-        series.slice(4, 2);
+        let _ = series.slice(-3, 4);
+        let _ = series.slice(-6, 2);
+        let _ = series.slice(4, 2);
     }
 
     #[test]
